#include "movieclip.h"	//Include the file wich containts all the headers we need

// ================================ MovieClip class ===============================
// ================================================================================

using namespace std;

list<MovieClip*> MovieClip::mc_list;
unsigned MovieClip::FPS = 30;
int MovieClip::fps;
MovieClip* MovieClip::roll = NULL;
MovieClip* MovieClip::press = NULL;
bool MovieClip::LBD = false;
int MovieClip::currentlayer = 0;

MovieClip::MovieClip()
{
	totalframes = currentframe = 0;
	visible = true;
	x = y = w = h = 0;
	xmouse = ymouse = xmouse_p = ymouse_p = 0;
	xdrag_m = ydrag_m = 0;
	depth = 0;
	layer = 0;
	alpha = 255;
	play_var = false;//start de mc stoped
	mouseover = false;
	prs = rls = false;
	drag = false;
	image = NULL;
	delimage = false;
	instance[0] = NULL;//empty instance
	//Triger functions
	onRollOver = onRollOut = onPress =  onRelease = onReleaseOutside = onEnterFrame = NULL;
	mc_list.push_back(this);
}

MovieClip::~MovieClip()
{
	if(totalframes)delete [] rectangles;
	if( image != NULL && delimage == true)SDL_FreeSurface( image );
	mc_list.remove(this);
}

void MovieClip::startFpsTimer()
{
	fps = SDL_GetTicks();
}

void MovieClip::RegulateFPS()
{
	if(SDL_GetTicks()-fps<1000/FPS) SDL_Delay( ( 1000 / MovieClip::FPS ) - (SDL_GetTicks()-fps) ); 
}

void MovieClip::Render(SDL_Surface*& screen)
{
	list<MovieClip*>::iterator it;
	it = mc_list.end();
	do
	{
		it--;
		if((*it)->visible && ( (*it)->layer == currentlayer || (*it)->layer == 0) )
		{
			(*it)->UpdateData();
			(*it)->ReactData();
		}
		else (*it)->Reset();
	}while(it != mc_list.begin());
	for ( it = mc_list.begin(); it != mc_list.end(); it++ )
	{
		(*it)->UpdateData();
		(*it)->ReactData();
		(*it)->RenderFrame(screen);
		if((*it)->play_var == 1)(*it)->NextFrame();//Next Frame
		else if((*it)->play_var == -1)(*it)->PreviousFrame();//Previous Frame
	}
}

void MovieClip::Reset()
{
	if(roll==this)roll = NULL;
	if(press==this)press = NULL;
	drag = false;
	play_var = 0;//start de mc stoped
	mouseover = false;
	prs = rls = false;
}

bool MovieClip::Init(const char *path, int nr_of_frames, int W, int H, int WIDTH, int dep)
{
	SDL_Surface *temp = NULL;
	 
	temp = IMG_Load(path);
	if (temp == NULL){
		printf("Unable to load image: %s\n%s\n", path, SDL_GetError());
		return false;
	}
	
	image = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

	currentframe = 0;
	if(totalframes)delete [] rectangles;
	totalframes = nr_of_frames;
	rectangles = new SDL_Rect[ totalframes ];

	int x2, y2 = x2 = 0;

	for(int i = 0; i < nr_of_frames; i++)
	{
		SetFrame(i, x2, y2, W, H);
		x2 += W;
		if( x2 >= WIDTH)
		{
			x2 = 0;
			y2 += H;
		}
	}

	x = 0;
	y = 0;
	h = H;
	w = W;
	
	SetDepth(dep);
	layer = currentlayer;
	delimage = true;

	return true;
}

bool MovieClip::InitFrom(MovieClip *mc, int dep)
{
	image = mc->image;

	currentframe = 0;
	if(totalframes)delete [] rectangles;
	totalframes = mc->totalframes;
	rectangles = new SDL_Rect[ totalframes ];

	for(int i = 0; i < totalframes; i++)
	{
		rectangles[i] = mc->rectangles[i];
	}

	x = 0;
	y = 0;
	h = mc->h;
	w = mc->w;
	
	SetDepth(dep);
	layer = currentlayer;
	delimage = false;

	return true;
}

void MovieClip::Clean()
{
	list<MovieClip*>::iterator it;
	for ( it = mc_list.begin(); it != mc_list.end(); it++ )
	{
		(*it)->visible = false;
		if( (*it)->image != NULL )
		{
			SDL_FreeSurface( (*it)->image );
		}
	}
	mc_list.clear();
}

MovieClip* MovieClip::GetByInstance(const char *str)
{
	list<MovieClip*>::iterator it;
	for ( it = mc_list.begin(); it != mc_list.end(); it++ )
		if(strcmp((*it)->instance, str) == 0)return (*it);
	return NULL;
}

void MovieClip::SetDepth(unsigned int Depth)
{
	depth = Depth;
	mc_list.sort(orderByDepth);
}

int MovieClip::currentFrame()
{
	return currentframe;
}

int MovieClip::totalFrames()
{
	return totalframes;
}

void MovieClip::ShowLayer(int Layer)
{
	currentlayer = Layer;
}

void MovieClip::SetLayer(int Layer)
{
	layer = Layer;
}

void MovieClip::setInstance(const char *str)
{
	strcpy(instance, str);
}

SDL_Rect* MovieClip::GetFrame(void)
{
	// Pointer to frame
	return &rectangles[ currentframe ];
}

void MovieClip::SetFrame( int frame, int x, int y, int w, int h)
{
	// setup the location and dimensions of a frame
	rectangles[ frame ].x = x;
	rectangles[ frame ].y = y;
	rectangles[ frame ].w = w;
	rectangles[ frame ].h = h;
}

void MovieClip::SetFrame( int frame, SDL_Rect* rect)
{
	// setup the location and dimensions of a frame
	rectangles[ frame ].x = rect->x;
	rectangles[ frame ].y = rect->y;
	rectangles[ frame ].w = rect->w;
	rectangles[ frame ].h = rect->h;
}

void MovieClip::NextFrame(void)
{
	currentframe++; // advance one frame
	if(currentframe >= totalframes) currentframe = 0; // if we go over the top set to zero
	return;
}

void MovieClip::PreviousFrame(void)
{
	currentframe--; // reverse one frame
	if(currentframe < 0) currentframe = totalframes - 1; // if we go below zero set to last frame
	return;
}

void MovieClip::setXY(int X, int Y)
{
	x = X;
	y = Y;
}

void MovieClip::play()
{
	play_var = 1;
}

void MovieClip::stop()
{
	play_var = 0;
}

void MovieClip::playReverse()
{
	play_var = -1;
}

void MovieClip::gotoAndPlay(int frame)
{
	if(frame < totalframes)currentframe = frame;
	else currentframe = totalframes-1;
	play_var = 1;
}

void MovieClip::gotoAndStop(int frame)
{
	if(frame < totalframes)currentframe = frame;
	else currentframe = totalframes-1;
	play_var = 0;
}

void MovieClip::RenderFrame(SDL_Surface*& screen)
{
	if( visible && (layer == currentlayer || layer == 0))
	{
		position.x = x;
		position.y = y;
		position.w = w;
		position.h = h;
		//SDL_SetAlpha( image, SDL_SRCALPHA, alpha ); disabled
		SDL_BlitSurface( image, GetFrame(), screen, &position );
	}
}

bool MovieClip::orderByDepth(MovieClip *a, MovieClip *b)
{
	if(a->depth > b->depth)return false;
	return true;
}

